@@ -10,12 +10,21 @@
 #	pragma hdrstop
 #endif
 
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #define _CRT_SECURE_NO_WARNINGS 1
+#endif
 
 const char gLibNcFTPVersion[] = kLibraryVersion;
 
 #ifdef NO_SIGNALS
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-const-variable"
+#endif
 static const char gNoSignalsMarker[] = "@(#) LibNcFTP - NO_SIGNALS";
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 #else
 
 static int gGotSig = 0;
@@ -29,7 +38,14 @@ static jmp_buf gCancelConnectJmp;
 
 
 #ifndef lint
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunused-const-variable"
+#endif
 static const char gCopyright[] = "@(#) LibNcFTP Copyright 1995-2001, by Mike Gleason.  All rights reserved.";
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 #endif
 
 #ifdef HAVE_LIBSOCKS5
@@ -208,7 +224,7 @@ OpenControlConnection(const FTPCIPtr cip, char *host, unsigned int port)
 		cip->servCtlAddr.sin_addr.s_addr = ip_address.s_addr;
 	} else {
 		hpok = 1;
-		cip->servCtlAddr.sin_family = hp.h_addrtype;
+		cip->servCtlAddr.sin_family = (sa_family_t) hp.h_addrtype;
 		/* We'll fill in the rest of the structure below. */
 	}
 	
@@ -544,7 +560,7 @@ OpenControlConnection(const FTPCIPtr cip, char *host, unsigned int port)
 	if ((hpok == 0) || (hp.h_name == NULL))
 		(void) STRNCPY(cip->actualHost, fhost);
 	else
-		(void) STRNCPY(cip->actualHost, (char *) hp.h_name);
+		(void) STRNCPY(cip->actualHost, (const char *) hp.h_name);
 
 	/* Read the startup message from the server. */	
 	rp = InitResponse();
@@ -978,6 +994,7 @@ OpenDataConnection(const FTPCIPtr cip, int mode)
 	int setsbufs;
 	size_t rbs, sbs;
 	int passiveAttemptsRemaining = cip->maxNumberOfSuccessivePASVAttempts;
+	char servDataAddrStr[64];
 
 	/* Before we can transfer any data, and before we even ask the
 	 * remote server to start transferring via RETR/NLST/etc, we have
@@ -1085,6 +1102,7 @@ tryPort:
 				goto bad;
 			}
 			FTPFixServerDataAddr(cip);
+			AddrToAddrStr(servDataAddrStr, sizeof(servDataAddrStr), &cip->servDataAddr, 0, NULL);
 
 #ifdef HAVE_LIBSOCKS
 			cip->ourDataAddr.sin_port = 0;
@@ -1105,13 +1123,14 @@ tryPort:
 
 			if (result == kTimeoutErr) {
 				if (mode == kFallBackToSendPortMode) {
-					FTPLogError(cip, kDontPerror, "Data connection timed out.\n");
+					FTPLogError(cip, kDontPerror, "Data connection to %s timed out.\n", servDataAddrStr);
 					if (passiveAttemptsRemaining == 0) {
-						FTPLogError(cip, kDontPerror, "Falling back to PORT instead of PASV mode.\n");
 						(void) DisposeSocket(dataSocket);
 						dataSocket = kClosedFileDescriptor;
 						if (cip->hasPASV == kCommandAvailabilityUnknown)
 							cip->hasPASV = kCommandNotAvailable;
+						if (cip->hasPASV == kCommandNotAvailable)
+							FTPLogError(cip, kDontPerror, "Falling back to PORT instead of PASV mode.\n");
 						goto tryPort2;
 					}
 				} else {
@@ -1145,19 +1164,21 @@ tryPort:
 						goto bad;
 				}
 				if ((mode == kFallBackToSendPortMode) && (passiveAttemptsRemaining == 0)) {
-					FTPLogError(cip, kDoPerror, "connect failed.\n");
-					FTPLogError(cip, kDontPerror, "Falling back to PORT instead of PASV mode.\n");
+					FTPLogError(cip, kDoPerror, "connect to %s failed.\n", servDataAddrStr);
 					(void) DisposeSocket(dataSocket);
 					dataSocket = kClosedFileDescriptor;
 					if (cip->hasPASV == kCommandAvailabilityUnknown)
 						cip->hasPASV = kCommandNotAvailable;
+					if (cip->hasPASV == kCommandNotAvailable)
+						FTPLogError(cip, kDontPerror, "Falling back to PORT instead of PASV mode.\n");
 					goto tryPort2;
 				}
-				FTPLogError(cip, kDoPerror, "connect failed.\n");
+				FTPLogError(cip, kDoPerror, "connect to %s failed.\n", servDataAddrStr);
 				result = kErrConnectDataSocket;
 				cip->errNo = kErrConnectDataSocket;
 			} else {
 				/* Success, break loop because we do not need to do PASV again. */
+				PrintF(cip, "Connected to %s for PASV.\n", servDataAddrStr);
 				break;
 			}
 
