@@ -10,7 +10,11 @@
 #	pragma hdrstop
 #endif
 
+#include "io_sendfile.h"
+
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #define _CRT_SECURE_NO_WARNINGS 1
+#endif
 
 #ifndef O_BINARY
 	/* Needed for platforms using different EOLN sequence (i.e. DOS) */
@@ -21,6 +25,60 @@
 #	endif
 #endif
 
+typedef struct MyDoSendfileProgressInfo {
+	DoSendfileProgressInfo d;
+	longest_int osize;
+	FTPCIPtr cip;
+} MyDoSendfileProgressInfo;
+
+
+
+
+static int
+MyDoSendfileProgressProc(DoSendfileProgressInfo *const dspip)
+{
+	MyDoSendfileProgressInfo *mp = (MyDoSendfileProgressInfo *) dspip;
+	FTPCIPtr cip;
+
+	if (mp == NULL) return 0;
+	cip = mp->cip;
+	if (dspip->current > 0)		/*  Will be 0 if DoSendfile wasn't using its progress meter */
+		cip->bytesTransferred = dspip->current;
+	FTPUpdateIOTimer(cip);
+	return 1;
+}	/* MyDoSendfileProgressProc */
+
+
+
+
+static void
+MyDoSendfileErrPrintF(DoSendfileParamsPtr dspp, const char *const fmt, ...)
+{
+	va_list ap;
+	MyDoSendfileProgressInfo *mp;
+	FTPCIPtr cip;
+	
+	if (dspp != NULL) {
+		mp = (MyDoSendfileProgressInfo *) dspp->dspip;
+		cip = mp->cip;
+		if (cip != NULL) {
+			va_start(ap, fmt);
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wformat-nonliteral"
+#endif
+            PrintF(cip, fmt, ap);
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+			va_end(ap);
+		}
+	}
+}	/* DoSendfileErrPrintF */
+
+
+
+
 static int
 FTPASCIILocalFileSeek(const int fd, const longest_int howMuchToSkip, char *const inbuf, const size_t bufsize)
 {
@@ -145,7 +203,9 @@ FTPPutOneF(
 	const char *volatile tmpsfx,
 	const int resumeflag,
 	const int deleteflag,
-	const FTPConfirmResumeUploadProc resumeProc)
+	const FTPConfirmResumeUploadProc resumeProc,
+	const time_t batchStartTime,
+	const time_t origLmtime)
 {
 	char *buf, *cp;
 	const char *cmd;
@@ -172,6 +232,13 @@ FTPPutOneF(
 	int skiprc;
 	size_t skipbufsize;
 	longstring cmdStr;
+	MyDoSendfileProgressInfo dspi;
+	DoSendfileParams dsp;
+
+	if (cip == NULL)
+		return (kErrBadParameter);
+	if ((memcmp(cip->magic, kLibraryMagic, kLibraryMagicLen) != 0) || (memcmp(cip->tailMagic, kLibraryMagic, kLibraryMagicLen) != 0))
+		return (kErrBadMagic);
 	
 	if (cip->buf == NULL) {
 		FTPLogError(cip, kDoPerror, "Transfer buffer not allocated.\n");
@@ -206,6 +273,14 @@ FTPPutOneF(
 		localsize = FTPLocalASCIIFileSize(file, cip->buf, cip->bufSize);
 	}
 
+	cip->usingSendfile = 0;
+	if ((cip->useSendfile != 0) && (DoSendfileAvailable() != 0))  {
+		/* Don't bother with small files. */
+		if (localsize >= 65536) {
+			cip->usingSendfile = 1;
+		}
+	}
+
 	/* For Put, we can't recover very well if it turns out restart
 	 * didn't work, so check beforehand.
 	 */
@@ -244,7 +319,6 @@ FTPPutOneF(
 		if (appendflag == kAppendYes) {
 			zaction = kConfirmResumeProcSaidAppend;
 		} else if (
-		/*		(cip->hasREST == kCommandNotAvailable) || We can now try to use APPE when REST is not availble. */
 		/*		(xtype != kTypeBinary) || We can now resume in ASCII too. */
 				(fstatrc < 0)
 		) {
@@ -304,8 +378,41 @@ FTPPutOneF(
 				/* Already sent file, done. */
 				zaction = kConfirmResumeProcSaidSkip; 
 				sameAsRemote = 1;
-			} else if ((startPoint != kSizeUnknown) && (localsize > startPoint)) {
-				zaction = kConfirmResumeProcSaidResume; 
+			} else if ((startPoint != kSizeUnknown) && (localsize > startPoint) && ((cip->hasREST == kCommandAvailable) || (cip->hasAPPE == kCommandAvailable))) {
+				if (batchStartTime != 0) {
+					/* If we had previously tried to upload this file,
+					 * then batchStartTime denotes the time we first
+					 * started trying.
+					 *
+					 * An incomplete upload should result in the remote
+					 * file timestamp that is newer than this time.
+					 */
+					if ((mdtm != kModTimeUnknown) && (mdtm >= batchStartTime)) {
+						/* If the remote file's timestamp looks recent, it should have the
+						 * the timestamp when the last block was written to the file.
+						 * If we know we started an upload prior to when this last block was written,
+						 * then we know this file didn't finish and we can resume it.
+						 */
+						if ((origLmtime != 0) && (origLmtime != st.st_mtime)) {
+							/* Local changed since last time. */
+							zaction = kConfirmResumeProcSaidOverwrite; 
+						} else if (st.st_mtime > (mdtm + 1)) {
+							/* Local file is newer than remote */
+							zaction = kConfirmResumeProcSaidOverwrite; 
+						} else {
+							/* Local file is older than remote */
+							zaction = kConfirmResumeProcSaidResume; 
+						}
+					} else {
+						zaction = kConfirmResumeProcSaidOverwrite; 
+					}
+				} else if ((mdtm != kModTimeUnknown) && (st.st_mtime > (mdtm + 1))) {
+					/* Local file is newer than remote */
+					zaction = kConfirmResumeProcSaidOverwrite; 
+				} else {
+					/* Local file is older than remote */
+					zaction = kConfirmResumeProcSaidResume; 
+				}
 			} else {
 				zaction = kConfirmResumeProcSaidOverwrite; 
 			}
@@ -343,6 +450,7 @@ FTPPutOneF(
 				}
 				return (kNoErr);
 			} else if (xtype == kTypeAscii) {
+				cip->usingSendfile = 0;
 				skipbufsize = cip->bufSize;
 				if ((fdtouse >= 0) && (skipbufsize > 4096))
 					skipbufsize = 4096;
@@ -351,7 +459,7 @@ FTPPutOneF(
 				if (skiprc == -2) {
 					cip->errNo = kErrAsciiSeekErr;
 					return (cip->errNo);
-				} else if (skiprc == -1) {
+				} else if ((cip->hasAPPE != kCommandNotAvailable) && (skiprc == -1)) {
 					/* Overwrite */
 					cip->startPoint = startPoint = 0;
 				} else {
@@ -363,6 +471,15 @@ FTPPutOneF(
 					zaction = kConfirmResumeProcSaidAppend;
 					cip->startPoint = startPoint = 0;
 				}
+			} else if (cip->hasREST == kCommandNotAvailable) {
+				/* Overwrite */
+				cip->startPoint = startPoint = 0;
+			} else if (cip->usingSendfile != 0) {
+				/* Do not set the file position.
+				 * sendfile() will be able to start from the offset
+				 * without seeking.
+				 */
+				cip->startPoint = startPoint;
 			} else if (Lseek(fd, startPoint, SEEK_SET) != -1) {
 				cip->startPoint = startPoint;
 			}
@@ -419,7 +536,7 @@ FTPPutOneF(
 		 *
 		 * So now we have to undo our seek.
 		 */
-		if (Lseek(fd, 0, SEEK_SET) != 0) {
+		if ((cip->usingSendfile == 0) && (Lseek(fd, 0, SEEK_SET) != 0)) {
 			cip->errNo = kErrLseekFailed;
 			if (fdtouse < 0) {
 				(void) close(fd);
@@ -440,8 +557,10 @@ FTPPutOneF(
 	}
 	cip->lname = file;	/* could be NULL */
 	cip->rname = odstfile;
-	if (fdtouse >= 0)
+	if (fdtouse >= 0) {
 		cip->useProgressMeter = 0;
+		cip->usingSendfile = 0;
+	}
 	FTPStartIOTimer(cip);
 
 	/* Note: On Windows, we don't have to do anything special
@@ -451,6 +570,7 @@ FTPPutOneF(
 
 	if (xtype == kTypeAscii) {
 		/* ascii */
+		cip->usingSendfile = 0;
 
 		if (cip->asciiTranslationMode == kAsciiTranslationModeNone) {
 			/* Warning: this is really just for testing FTP server software.
@@ -575,8 +695,50 @@ FTPPutOneF(
 				}
 			}
 		}
+	} else if (cip->usingSendfile != 0) {
+		/* binary, but use sendfile() */
+		DoSendfileProgressInfoInit((DoSendfileProgressInfo *) &dspi, sizeof(dspi));
+		DoSendfileParamsInit(&dsp, sizeof(dsp));
+		dsp.dspip = (DoSendfileProgressInfo *) &dspi;
+		dsp.dsppp = MyDoSendfileProgressProc;
+		dsp.dseppp = MyDoSendfileErrPrintF;
+		dspi.cip = cip;
+		dsp.buf = buf;
+		dsp.bufSize = bufSize;
+		dsp.handleSIGINT = 0;
+		dsp.xferTimeout = cip->xferTimeout;
+
+		if ((cip->progress != (FTPProgressMeterProc) 0) && (cip->useProgressMeter != 0))
+			dsp.nonBlockingOutput = 1;
+
+#ifdef O_NONBLOCK
+		if (dsp.nonBlockingOutput != 0) {
+			if (fcntl(cip->dataSocket, F_SETFL, O_NONBLOCK) < 0) {
+				FTPLogError(cip, kDoPerror, "Could not set nonblocking mode on output socket.\n");
+				dsp.nonBlockingOutput = 0;
+			}
+		}
+
+		if (dsp.nonBlockingInput != 0) {
+			if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
+				FTPLogError(cip, kDoPerror, "Could not set nonblocking mode on input file.\n");
+				dsp.nonBlockingInput = 0;
+			}
+		}
+#else
+		dsp.nonBlockingOutput = 0;
+		dsp.nonBlockingInput = 0;
+#endif	/* O_NONBLOCK */
+
+		PrintF(cip, "Using sendfile for " PRINTF_LONG_LONG " bytes.\n", localsize);
+		result = DoSendfile(fd, localsize, cip->dataSocket, &dspi.osize, &dsp);
+		if (result == 0) {
+			cip->bytesTransferred = dspi.osize;
+			MyDoSendfileProgressProc((DoSendfileProgressInfo *) &dspi);
+		}
 	} else {
 		/* binary */
+		cip->usingSendfile = 0;
 		for (;;) {
 			cp = buf;
 			nread = read(fd, cp, (read_size_t) bufSize);
@@ -645,6 +807,10 @@ brk:
 		 */
 		cip->numUploads++;
 
+		if (strncmp(cmd, "APPE", 4) == 0) {
+			cip->hasAPPE = kCommandAvailable;
+		}
+
 		if ((tmppfx[0] != '\0') || (tmpsfx[0] != '\0')) {
 			if ((result = FTPRename(cip, dstfile, odstfile)) < 0) {
 				/* May fail if file was already there,
