@@ -10,6 +10,11 @@
 #	pragma hdrstop
 #endif
 
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#	define _CRT_SECURE_NO_WARNINGS 1
+#	pragma warning(disable : 4706)	// warning C4706: assignment within conditional expression
+#endif
+
 static const char *rwx[9] = { "---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx", NULL };
 
 #if 0
@@ -280,8 +285,9 @@ UnLslRLine(	char *const line,
 		int *plugend)
 {
 	char *cp;
-	int mon = 0, dd = 0, hr = 0, min = 0, year = 0;
-	char *monstart, *ddstart, *hrstart, *minstart, *yearstart;
+	int mon = 0, day = 0, hr = 0, min = 0, year = 0, sec = 0;
+	int haveIsoHHMMSS = -1;
+	char *monstart, *daystart, *hrstart, *minstart, *yearstart, *secstart = NULL;
 	char *linktostart, *filestart = NULL;
 	char *sizestart;
 	char *pe;
@@ -303,6 +309,7 @@ UnLslRLine(	char *const line,
 -rw-rw-r--   1 gleason  sysdevzz     1829 Jul 7   1996 README
 -rw-rw-r--   1 gleason  sysdevzz     1829 Jul  7 1996  README
 -rw-rw-r--   1 gleason  sysdevzz     1829 Jul 7  1996  README
+drwxrwxrwx   1 pvr pvr              45056 Jan  1 2000 DataFiles
          *
 	 *------------------------------^
 	 *                              0123456789012345
@@ -325,7 +332,7 @@ UnLslRLine(	char *const line,
 			&& (isspace((int) cp[8]))
 		) {
 			monstart = cp + 2;
-			ddstart = cp + 6;
+			daystart = cp + 6;
 			if (	((isspace((int) cp[9])) || (isdigit((int) cp[9])))
 				&& (isdigit((int) cp[10]))
 				&& (isdigit((int) cp[11]))
@@ -341,12 +348,18 @@ UnLslRLine(	char *const line,
 				filestart = cp + 15;
 				cp[1] = '\0';	/* end size */
 				cp[5] = '\0';	/* end mon */
-				cp[8] = '\0';	/* end dd */
-				cp[14] = '\0';	/* end year */
+				cp[8] = '\0';	/* end day */
+				if (! isspace((int) filestart[-1])) {
+					if (isspace((int) filestart[-2])) {
+						filestart--;
+					}
+				}
+				filestart[-1] = '\0';	/* end year */
 				mon = LsMonthNameToNum(monstart);
-				dd = atoi(ddstart);
+				day = atoi(daystart);
 				hr = 23;
 				min = 59;
+				sec = 59;
 				year = atoi(yearstart);
 
 				pe = cp;
@@ -372,13 +385,14 @@ UnLslRLine(	char *const line,
 				filestart = cp + 15;
 				cp[1] = '\0';	/* end size */
 				cp[5] = '\0';	/* end mon */
-				cp[8] = '\0';	/* end dd */
+				cp[8] = '\0';	/* end day */
 				cp[11] = '\0';	/* end hr */
 				cp[14] = '\0';	/* end min */
 				mon = LsMonthNameToNum(monstart);
-				dd = atoi(ddstart);
+				day = atoi(daystart);
 				hr = atoi(hrstart);
 				min = atoi(minstart);
+				sec = 0;
 				year = 0;
 
 				pe = cp;
@@ -389,6 +403,81 @@ UnLslRLine(	char *const line,
 				*plugend = (int) (pe - line) + 1;
 				break;
 			}
+		} else if (	(isdigit((int) *cp))
+			&& (isspace((int) cp[1]))
+
+			&& (isdigit((int) cp[2]))
+			&& (isdigit((int) cp[3]))
+			&& (isdigit((int) cp[4]))
+			&& (isdigit((int) cp[5]))
+			&& (cp[6] == '-')
+			&& (isdigit((int) cp[7]))
+			&& (isdigit((int) cp[8]))
+			&& (cp[9] == '-')
+			&& (isdigit((int) cp[10]))
+			&& (isdigit((int) cp[11]))
+
+			&& (isspace((int) cp[12]))
+
+			&& (isdigit((int) cp[13]))
+			&& (isdigit((int) cp[14]))
+			&& (cp[15] == ':')
+			&& (isdigit((int) cp[16]))
+			&& (isdigit((int) cp[17]))
+
+			&& (
+				((haveIsoHHMMSS = isspace((int) cp[18]))) ||
+				(
+					   (isdigit((int) cp[19]))
+					&& (isdigit((int) cp[20]))
+					&& (isspace((int) cp[21]))
+					&& ((haveIsoHHMMSS = (int) cp[18]) == ':')
+				)
+
+			)
+		) {
+			/* "YYYY-mm-dd HH:MM" or "YYYY-mm-dd HH:MM:SS" form */
+/* drwxr-xr-x   4 ftpuser  ftpusers  136 2008-03-12 23:38 beta
+                                       0123456789012345678901234567890123456789
+*/
+			yearstart = cp + 2;
+			cp[6] = '\0';	/* end year */
+
+			monstart = cp + 7;
+			cp[9] = '\0';	/* end month */
+
+			daystart = cp + 10;
+			cp[12] = '\0';	/* end day */
+
+			hrstart = cp + 13;
+			cp[15] = '\0';	/* end hr */
+
+			minstart = cp + 16;
+			cp[18] = '\0';	/* end min */
+
+			if (haveIsoHHMMSS == ':') {
+				secstart = cp + 19;
+				cp[21] = '\0';	/* end sec */
+				sec = atoi(secstart);
+				filestart = cp + 22;
+			} else {
+				filestart = cp + 19;
+				sec = 0;
+			}
+
+			mon = atoi(monstart) - 1;
+			day = atoi(daystart);
+			hr = atoi(hrstart);
+			min = atoi(minstart);
+			year = atoi(yearstart);
+
+			pe = cp;
+			while (isdigit((int) *pe))
+				pe--;
+			while (isspace((int) *pe))
+				pe--;
+			*plugend = (int) (pe - line) + 1;
+			break;
 		}
 	}
 
@@ -414,27 +503,31 @@ UnLslRLine(	char *const line,
 	if (ftime != NULL) {
 		(void) memset(&ftm, 0, sizeof(struct tm));
 		ftm.tm_mon = mon;
-		ftm.tm_mday = dd;
+		ftm.tm_mday = day;
 		ftm.tm_hour = hr;
 		ftm.tm_min = min;
+		ftm.tm_sec = sec;
 		ftm.tm_isdst = -1;
 		if (year == 0) {
 			/* We guess the year, based on what the
-			 * current year is.  We know the file
-			 * on the remote server is either less
-			 * than six months old or less than
-			 * one hour into the future.
+			 * current year is.
+			 *
+			 * UNIX ls would always print the year
+			 * if the file was older than 6 months,
+			 * but many non-UNIX based servers do
+			 * not honor that convention, so we
+			 * now assume that if the year was not
+			 * given it implies the current year,
+			 * unless the file is up to two days 
+			 * into the future.
 			 */
 			ftm.tm_year = thisyear - 1900;
 			*ftime = mktime(&ftm);
 			if (*ftime == (time_t) -1) {
 				/* panic */
-			} else if (*ftime > (now + (15552000L + 86400L))) {
+			} else if (*ftime > (now + (2 * 86400L))) {
 				--ftm.tm_year;
 				*ftime = mktime(&ftm);
-			} else if (*ftime < (now - (15552000L + 86400L))) {
-				++ftm.tm_year;
-				*ftime = mktime(&ftm);
 			}
 		} else {
 			ftm.tm_year = year - 1900;
@@ -854,8 +947,13 @@ UnMlsD(const FTPCIPtr cip, FTPFileInfoListPtr filp, FTPLineListPtr llp)
 				m1 = (mli.mode & 00700) >> 6;
 				m2 = (mli.mode & 00070) >> 3;
 				m3 = (mli.mode & 00007);
-			}
-			if (mli.perm[0] != '\0') {
+			} else if (mli.perm[0] != '\0') {
+				/* Note that if both UNIX.mode and
+				 * perm are avaiable, we will use
+				 * UNIX.mode.  This is probably
+				 * not what the RFC would like
+				 * us to do.
+				 */
 				m3 = 0;
 				if (fi.type == 'd') {
 					if (strchr(mli.perm, 'e') != NULL) {
@@ -891,12 +989,15 @@ UnMlsD(const FTPCIPtr cip, FTPFileInfoListPtr filp, FTPLineListPtr llp)
 				if (mli.group[0] != '\0') {
 #ifdef HAVE_SNPRINTF
 					snprintf(og, sizeof(og) - 1,
+                             "   %-8.8s %s",
+                             mli.owner, mli.group
+                             );
 #else
 					sprintf(og,
+                            "   %-8.8s %s",
+                            mli.owner, mli.group
+                            );
 #endif	/* HAVE_SNPRINTF */
-						"   %-8.8s %s",
-						mli.owner, mli.group
-					);
 					STRNCAT(plug, og);
 				} else {
 					STRNCAT(plug, "   ");
