@@ -1,6 +1,6 @@
 /* bookmark.c 
  *
- * Copyright (c) 1992-2004 by Mike Gleason.
+ * Copyright (c) 1992-2016 by Mike Gleason.
  * All rights reserved.
  * 
  */
@@ -174,7 +174,7 @@ ParseHostLine(char *line, BookmarkPtr bmp)
 				if ((n1 >= 0) && (n2 >= 0)) {
 					n = (n1 << 4) | n2;
 					if (d < tokenend)
-						*(unsigned char *)d++ = (unsigned int) n;
+						*(unsigned char *)d++ = (unsigned char) n;
 				}
 				s += 3;
 			} else {
@@ -302,7 +302,10 @@ OpenBookmarkFile(int *numBookmarks0)
 		return NULL;		/* Okay to not have one yet. */
 	}
 	
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#else
 	(void) chmod(pathName, 00600);
+#endif
 	if (FGets(line, sizeof(line), fp) == NULL) {
 		(void) fprintf(stderr, "%s: invalid format.\n", pathName);
 		(void) fclose(fp);
@@ -404,16 +407,22 @@ GetBookmark(const char *const bmabbr, Bookmark *bmp)
 	int exactMatch = 0;
 	size_t bmabbrLen;
 	char *cp;
+	char bmabbrtrunc[sizeof(bmp->bookmarkName)];
 
 	fp = OpenBookmarkFile(NULL);
 	if (fp == NULL)
 		return (-1);
 
+	memset(&byHostName, 0, sizeof(Bookmark));
+	memset(&byHostAbbr, 0, sizeof(Bookmark));
+	memset(&byBmAbbr, 0, sizeof(Bookmark));
+
+	STRNCPY(bmabbrtrunc, bmabbr);
 	bmabbrLen = strlen(bmabbr);
 	while (FGets(line, sizeof(line), fp) != NULL) {
 		if (ParseHostLine(line, bmp) < 0)
 			continue;
-		if (ISTREQ(bmp->bookmarkName, bmabbr)) {
+		if (ISTREQ(bmp->bookmarkName, bmabbrtrunc)) {
 			/* Exact match, done. */
 			byBmNameFlag = bmabbrLen;
 			exactMatch = 1;
@@ -538,7 +547,7 @@ LoadBookmarkTable(void)
 	if ((nb != gNumBookmarks) && (gBookmarkTable != NULL)) {
 		/* Re-loading the table from disk. */
 		gBookmarkTable = (Bookmark *) realloc(gBookmarkTable, (size_t) (nb + 1) * sizeof(Bookmark));
-		memset(gBookmarkTable, 0, (nb + 1) * sizeof(Bookmark));
+		memset(gBookmarkTable, 0, (size_t) (nb + 1) * sizeof(Bookmark));
 	} else {
 		gBookmarkTable = calloc((size_t) (nb + 1), (size_t) sizeof(Bookmark));
 	}
@@ -579,7 +588,7 @@ BmEscapeTok(char *dst, size_t dsize, char *src)
 			/* These need to be escaped. */
 			if ((dst + 1) < dlim) {
 				*dst++ = '\\';
-				*dst++ = c;
+				*dst++ = (char) c;
 			}
 		} else if (!isprint(c)) {
 			/* Escape non-printing characters. */
@@ -589,7 +598,7 @@ BmEscapeTok(char *dst, size_t dsize, char *src)
 			}
 		} else {
 			if (dst < dlim)
-				*dst++ = c;
+				*dst++ = (char) c;
 		}
 	}
 	*dst = '\0';
@@ -651,6 +660,9 @@ SwapBookmarkFiles(void)
 
 	(void) OurDirectoryPath(path2, sizeof(path2), kBookmarkFileName);
 	(void) OurDirectoryPath(pathName, sizeof(pathName), kTmpBookmarkFileName);
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__) && !defined(getpid)
+#	define getpid _getpid
+#endif
 	(void) sprintf(pidStr, "-%u.txt", (unsigned int) getpid());
 	(void) STRNCAT(pathName, pidStr);
 
@@ -688,7 +700,10 @@ OpenTmpBookmarkFile(int nb)
 		perror(pathName);
 		return (NULL);
 	}
+#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+#else
 	(void) chmod(pathName, 00600);
+#endif
 	if (nb > 0) {
 		if (fprintf(outfp, "NcFTP bookmark-file version: %d\nNumber of bookmarks: %d\n", kBookmarkVersion, nb) < 0) {
 			(void) fprintf(stderr, "Could not save bookmark.\n");
@@ -852,6 +867,33 @@ DefaultBookmarkName(char *dst, size_t siz, char *src)
 
 
 
+int
+AddNewItemToBookmarkTable(void)
+{
+	int nb;
+	BookmarkPtr newTable, bmp;
+
+	if (gBookmarkTable == NULL)
+		return (-1);
+
+	nb = gNumBookmarks + 1;
+	
+	newTable = (BookmarkPtr) realloc(gBookmarkTable, (size_t) (nb) * sizeof(Bookmark));
+	if (newTable == NULL)
+		return (-1);
+
+	gBookmarkTable = newTable;
+	gNumBookmarks = nb;
+
+	bmp = &newTable[nb - 1];
+	SetBookmarkDefaults(bmp);
+
+	return (nb - 1);
+}	/* AddNewItemToBookmarkTable */
+
+
+
+
 void
 DisposeBookmarkTable(void)
 {
