@@ -1,6 +1,6 @@
 /* pref.c
  *
- * Copyright (c) 1992-2006 by Mike Gleason.
+ * Copyright (c) 1992-2016 by Mike Gleason.
  * All rights reserved.
  * 
  */
@@ -188,7 +188,8 @@ SetConnTimeout(const char *const val, FILE *const fp)
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gConnTimeout);
 	} else {
-		gConn.connTimeout = gConnTimeout = atoi(val);
+		gConn.connTimeout = (unsigned int) atoi(val);
+        gConnTimeout = atoi(val);
 	}
 }	/* SetConnTimeout */
 
@@ -200,7 +201,8 @@ SetCtrlTimeout(const char *const val, FILE *const fp)
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gCtrlTimeout);
 	} else {
-		gConn.ctrlTimeout = gCtrlTimeout = atoi(val);
+        gConn.ctrlTimeout = (unsigned int) atoi(val);
+        gCtrlTimeout = atoi(val);
 	}
 }	/* SetCtrlTimeout */
 
@@ -363,7 +365,8 @@ SetSOBufsize(const char *const val, FILE *const fp)
 		if (gSOBufsize <= 0)
 			(void) fprintf(fp, "%s", " (use system default)");
 	} else {
-		gConn.dataSocketRBufSize = gConn.dataSocketSBufSize = gSOBufsize = atoi(val);
+		gConn.dataSocketRBufSize = gConn.dataSocketSBufSize = (size_t) atoi(val);
+        gSOBufsize = atoi(val);
 	}
 }	/* SetSOBufsize */
 
@@ -376,7 +379,8 @@ SetXferTimeout(const char *const val, FILE *const fp)
 	if (fp != NULL) {
 		(void) fprintf(fp, "%d", gXferTimeout);
 	} else {
-		gConn.xferTimeout = gXferTimeout = atoi(val);
+		gConn.xferTimeout = (unsigned int) atoi(val);
+        gXferTimeout = atoi(val);
 	}
 }	/* SetXferTimeout */
 
