@@ -152,16 +152,16 @@ wg_AddCh(int c)
 			/* Just add a character to the end.  No need to do
 			 * a memory move for this.
 			 */
-			*gBufPtr = c;
+			*gBufPtr = (char) c;
 			gBufLen++;
 			wg_SetCursorPos(gBufPtr + 1);
 		} else {
 			/* Have to move characters after the cursor over one
 			 * position so we can insert a character.
 			 */
-			n = limit - gBufPtr;
+			n = (size_t) (limit - gBufPtr);
 			MEMMOVE(gBufPtr + 1, gBufPtr, n);
-			*gBufPtr = c;
+			*gBufPtr = (char) c;
 			gBufLen++;
 			wg_SetCursorPos(gBufPtr + 1);
 		}
@@ -189,10 +189,10 @@ wg_KillCh(int count)
 			/* Delete the characters before the character under the
 			 * cursor, and move everything after it back one.
 			 */
-			n = limit - gBufPtr;
+			n = (size_t) (limit - gBufPtr);
 			memcpy(gBufPtr - count, gBufPtr, n);
 		}
-		gBufLen -= count;
+		gBufLen -= (size_t) count;
 		wg_SetCursorPos(gBufPtr - count);	/* Does a --gBufPtr. */
 		gNeedUpdate = 1;
 		gChanged = 1;
@@ -237,7 +237,7 @@ wg_ForwardKillCh(void)
 			/* Nothing in front to delete. */
 			BEEP(1);
 		} else {
-			n = limit - gBufPtr - 1;
+			n = (size_t) (limit - gBufPtr) - 1;
 			memcpy(gBufPtr, gBufPtr + 1, n);
 			--gBufLen;
 			gNeedUpdate = 1;
